package tese.intervalos.base;

import tese.intervalos.Constantes;
import tese.intervalos.excecoes.ExcecaoFormatoNumeroInvalido;

/**
 * <p>Title: Intervalo</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class Intervalo {

  /**
   * atributos
   */
  protected Number limiteInferior;
  protected Number limiteSuperior;

  /**
   * contrutor padrao
   */
  public Intervalo() {
    this.limiteInferior = new Number();
    this.limiteSuperior = new Number();
  }

  /**
   * Contrutor com limites
   * @param limInf
   * @param limSup
   */
  public Intervalo(Number limInf, Number limSup ) {
    AtribuirLimites(limInf, limSup);
  }

  private void AtribuirLimites(Number limInf, Number limSup) {
    if (Number.comparar(limInf, limSup) <= 0) {
       this.limiteInferior = limInf;
       this.limiteSuperior = limSup;
    } else {
       this.limiteInferior = limSup;
       this.limiteSuperior = limInf;
    }
  }

  /**
   * Construtor com parametro unico
   * @param value
   */
  public Intervalo (Number n) {
    AtribuirIntervaloPontual(n);
  }

  private void AtribuirIntervaloPontual(Number n) {
    this.limiteInferior = n;
    this.limiteSuperior = n.clonar();
  }

  /**
   *
   * @param number
   */
  public Intervalo (String number){
    Number n = new Number(number);
    AtribuirIntervaloPontual(n);
  }

  public Intervalo (String number1, String number2) {
    Number n1 = new Number(number1);
    Number n2 = new Number(number2);
    AtribuirLimites(n1,n2);
  }

  /**
   * retirna o limite inferior de um intervalo
   * @return
   */
  public Number getLimiteInferior() {
    return limiteInferior;
  }

  /**
   * altera o limite inferior de um intervalo
   * @param limiteInferior
   */
  public void setLimiteInferior(Number limiteInferior) {
    this.limiteInferior = limiteInferior;
  }

  /**
   * retorna o limite superior de um intervalo
   * @return
   */
  public Number getLimiteSuperior() {
    return limiteSuperior;
  }

  /**
   * altera o limite superior de um intervalo
   * @param limiteSuperior
   */
  public void setLimiteSuperior(Number limiteSuperior) {
    this.limiteSuperior = limiteSuperior;
  }

  /**
   * calcula o diametro de um intervalo
   * @return
   */
  public Number getDiametro(){
    return Number.subtrair(this.limiteSuperior, this.limiteInferior);
  }

  /**
   * calcula o ponto medio de um intervalo
   * @return
   */
  public Number getPontoMedio(int precision){
    Number n = null;

      n = Number.dividir((Number.adicionar(this.limiteSuperior, this.limiteInferior)),
                          new Number("2"),false,precision);

    return n;
  }

  /**
   * Calcula a distancia de um intervalo a outro
   * @param b
   * @return
   */
  public static Number getDistancia(Intervalo a, Intervalo b) {
    return Number.max(Number.modulo(Number.subtrair(a.getLimiteInferior(),b.getLimiteInferior())),
                      Number.modulo(Number.subtrair(a.getLimiteSuperior(),b.getLimiteSuperior())));
  }

  /**
   * Multiplica um intervalo por um escalar
   * @param d
   */
  public void multiplicacoPorEscalar(Number d){
    Number mult1 = Number.multiplicar(this.limiteInferior,d);
    Number mult2 = Number.multiplicar(this.limiteSuperior,d);

    this.limiteInferior = Number.min(mult1, mult2);
    this.limiteSuperior = Number.max(mult1, mult2);
  }

  /**
   * imprime um Intervalo de Strings
   * @return
   */
  public String toString(){
    return "[" + limiteInferior + ", " + limiteSuperior + "]";
  }

  /**
   * calcula o valor absoluto de um intervalo que corresponde ao
   * maior mdulo entre os limites uperiores e inferiores
   * @return Number
   */
  public Number valorAbsoluto(){
    Number mls = Number.modulo(this.limiteSuperior);
    Number mli = Number.modulo(this.limiteInferior);
    Number retorno = Number.max(mls, mli);
    return retorno;
  }

  /**
   * verifica de um intervalo e igual a outro
   * @param intervalo
   * @return
   */
  public static boolean igualdade (Intervalo intervaloA, Intervalo intervaloB){
    return (Number.comparar(intervaloA.limiteInferior, intervaloB.limiteInferior) == 0 &&
        Number.comparar(intervaloA.limiteSuperior, intervaloB.limiteSuperior) == 0);
  }

  /**
   * Prints interval in scientific notation
   * @param precision number of symbols after the comma
   * @return interval in scientific notation
   */
  public String toScientificNotation(int precision) {
    return "[" + this.limiteInferior.toScientificNotation(precision) +
           " , " + this.limiteSuperior.toScientificNotation(precision) + "]";
  }
}