package tese.intervalos.base;

import java.util.StringTokenizer;
import java.util.HashSet;
import java.util.Hashtable;
import tese.intervalos.excecoes.ExcecaoFormatoNumeroInvalido;
import tese.intervalos.excecoes.ExcecaoDivisaoPorZero;
import java.io.*;
import java.math.BigDecimal;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class Number {
  //Constantes de sinal
  public static final int SINAL_POSITIVO = 1;
  public static final int SINAL_NEGATIVO = -1;
  public static final int SINAL_ZERO = 0;
  public static final char CARACTERE_ZERO = 48;

/*********************************************************************
                      Atributos de um numero
**********************************************************************/

  /**
   * Sinal do numero
   * Sinal positivo = 1
   * Sinal neutro = 0
   * Sinal negativo = -1
   */
  protected int sinal = SINAL_ZERO;

  /**
   * mantissa do numero, um String numerico
   */
  protected String mantissa = "0";

  /**
   * Sinal do expoente
   * Sinal positivo = 1
   * Sinal neutro = 0
   * Sinal negativo = -1
   */
  protected int sinalExpoente = SINAL_ZERO;

  /**
   * String numerico representando o expoente
   */
  protected String expoente = "";

  /**
   * Objeto que representa o denominador
   */
  protected Number denominador = null;

  /**************************************************************************
   *
   * Construtor da Classe
   *
   **************************************************************************/

  /**
   * Construtor que recebe os parametros
   * @param mantissa
   * @param sinal
   * @param sinalExpoente
   * @param expoente
   * @param denominador
   */
  private Number(String mantissa, int sinal, int sinalExpoente, String expoente,
                 Number denominador) throws ExcecaoDivisaoPorZero{
    this.sinal = sinal;
    this.mantissa = mantissa;
    this.sinalExpoente = sinalExpoente;
    this.expoente = expoente;
    this.denominador = denominador;

    /******************* regra de formacao **********************************/

      this.mantissa = Number.removeLeftZeros(mantissa);
      this.expoente = Number.removeLeftZeros(expoente);
      boolean ExcecaoDivisaoPorZero = false;
      if (this.denominador != null) {
        ExcecaoDivisaoPorZero = verificaExcecaoDivisaoPorZero(this.denominador);
      }

      if ( this.mantissa.equals("")) {
        this.mantissa = "0";
      }

      if (this.mantissa.equals("0")){

        //nao aconteceu divisao por zero,
        //pois zero dividido por qualquer numero diferente de zero
        //eh igual a zero
        if (!ExcecaoDivisaoPorZero){
          this.denominador = null;
        }
        this.sinal = Number.SINAL_ZERO;
        this.expoente = "0";
      }
      if (this.expoente.equals("0")){
        this.sinalExpoente = Number.SINAL_ZERO;
      }

      if (this.denominador != null) {
        simplificarDenominadores(this);
        eliminaExpoenteSinalDenominador(this);
        simplificarMantissaFracao(this);
      }

     /************************************************************************/
  }

   /**
    * Construtor do numero zero
    */
   public Number () {
     this.sinal = Number.SINAL_ZERO;
     this.mantissa = "0";
     this.sinalExpoente = Number.SINAL_ZERO;
     this.expoente = "0";
     this.denominador = null;
   }


   /**
    * @param number String representando um nmero.
    * @throws ExcecaoFormatoNumeroInvalido
    * @throws ExcecaoDivisaoPorZero
    */
   public Number (String number) throws ExcecaoFormatoNumeroInvalido, ExcecaoDivisaoPorZero{
     Number n = createNumber(number);
     this.mantissa = n.mantissa;
     this.sinal = n.sinal;
     this.sinalExpoente = n.sinalExpoente;
     this.expoente = n.expoente;
     this.denominador = n.denominador;
   }

  /**
   * Cria um objeto do tipo Number a partir de um String
   * @param num um string da forma
   *
   * @thorws ExcecaoFormatoNumeroInvalido se o String passado
   *         nao obedecer o padrao.
   * @return object Number
   */
  public static Number createNumber (String num)
      throws ExcecaoDivisaoPorZero {
    Number n = null;
    int sinal = Number.SINAL_ZERO;
    int sinalExpoente = Number.SINAL_ZERO;;
    String mantissa = "0";
    String mantissaExpoente = "0";
    Number denominador = null;
    boolean antesVirgula = true;
    int casasDecimaisDepoisVigula = 0;

    if (num == null || num.length() < 1){
      throw new ExcecaoFormatoNumeroInvalido(num);
    }
    String number = num;
    number = removeLeftZerosFormatacao(number);


    //define qual o sinal do numero
    if (number.charAt(0) == '-' ) {
      sinal = Number.SINAL_NEGATIVO;
      number = number.substring(1,number.length());
      if (number.charAt(0) == ',' ||
          number.charAt(0) == '.') {
          antesVirgula = false;

      }

    } else if (number.charAt(0) == '0') {
      sinal = Number.SINAL_ZERO;
      mantissa = "0";
      mantissaExpoente = "0";
      sinalExpoente = SINAL_ZERO;
      if (number.length() > 1) {
        if (number.charAt(1) == '/'){
          number = number.substring(2,number.length());
          denominador = createNumber(number);
          number = "";
        } else if (number.charAt(1) == 'E' || number.charAt(1) == 'e') {
          number = number.substring(1,number.length());
          String [] temp = null;
          try {
            temp = parseExpoente(number, mantissaExpoente, sinalExpoente);
          } catch (ExcecaoFormatoNumeroInvalido efi) {
            throw new ExcecaoFormatoNumeroInvalido(num);
          }
          number = temp[0];
          mantissaExpoente = temp [1];
          sinalExpoente = Integer.parseInt(temp[2]);

        } else if (!number.equals("")) {
        //deveriam ter acabado os caracteres que formam o numero
         throw new ExcecaoFormatoNumeroInvalido(num);
      }

        if (!number.equals("") && number.charAt(0) == '/'){
         number = number.substring(1,number.length());
         denominador = createNumber(number);
         number = "";
        } else if (!number.equals("")) {
        //deveriam ter acabado os caracteres que formam o numero
         throw new ExcecaoFormatoNumeroInvalido(num);
      }


      }

    } else if (number.charAt(0) == '+' ||
               number.charAt(0) == ',' ||
               number.charAt(0) == '.' ||
              (number.charAt(0) >= CARACTERE_ZERO && number.charAt(0) <='9')) {
      sinal = Number.SINAL_POSITIVO;


      if (number.charAt(0) == ',' ||
          number.charAt(0) == '.') {
          antesVirgula = false;
      }

      if (number.charAt(0) == '+') {
        number = number.substring(1,number.length());
      }

    } else {
      throw new ExcecaoFormatoNumeroInvalido(num);
    }

    if (sinal != SINAL_ZERO) {
      /*ler a mantissa*/
      if (!antesVirgula) {
        number = number.substring(1,number.length());
      }
      int i = 0;
      while (i < number.length() && number.charAt(i) >= CARACTERE_ZERO && number.charAt(i) <= '9') {
        i++;
      }
      //recebe os primeiros numeros da mantissa antes do ponto ou virgula
      mantissa = number.substring(0,i);
      number = number.substring(i,number.length());
      if (!antesVirgula) {
        casasDecimaisDepoisVigula = i;
      }
      i = 0;
      if (!number.equals("") && (number.charAt(i) == '.' || number.charAt(i) == ',')) {
        if (!antesVirgula){
          throw new ExcecaoFormatoNumeroInvalido(num);
        } else {
          //numeros apos a virgula ou ponto
          number = number.substring(1,number.length());
          while (i < number.length() && number.charAt(i) >= CARACTERE_ZERO && number.charAt(i) <= '9' ) {
            i++;
          }
          mantissa = mantissa + number.substring(0,i);
          casasDecimaisDepoisVigula = i;
          number = number.substring(i,number.length());
          i = 0;
        }
      } else if (!number.equals("") && (number.charAt(i) == 'E' || number.charAt(i) == 'e')) {
        String [] temp = null;
          try {
            temp = parseExpoente(number, mantissaExpoente, sinalExpoente);
          } catch (ExcecaoFormatoNumeroInvalido efi) {
            throw new ExcecaoFormatoNumeroInvalido(num);
          }
        number = temp[0];
        mantissaExpoente = temp [1];
        sinalExpoente = Integer.parseInt(temp[2]);
        i = 0;
      } else if (!number.equals("") && number.charAt(i) == '/') {
        //trata caso do denominador
        number = number.substring(1,number.length());
        denominador = createNumber(number);
        number = "";
      } else if (!number.equals("")){
        //nao tem nem ponto nem virgula nem indica o expoente, nem sinal da diviso
        throw new ExcecaoFormatoNumeroInvalido(num);
      }
      if (!number.equals("") && (number.charAt(i) == 'E' || number.charAt(i) == 'e')) {
         String [] temp = null;
          try {
            temp = parseExpoente(number, mantissaExpoente, sinalExpoente);
          } catch (ExcecaoFormatoNumeroInvalido efi) {
            throw new ExcecaoFormatoNumeroInvalido(num);
          }
        number = temp[0];
        mantissaExpoente = temp [1];
        sinalExpoente = Integer.parseInt(temp[2]);
        i = 0;

      } else if (!number.equals("") && number.charAt(i) == '/') {
        //trata caso do denominador
        number = number.substring(1,number.length());
        denominador = createNumber(number);
        number = "";
      } else if (!number.equals("")) {
        //deveriam ter acabado os caracteres que formam o numero
         throw new ExcecaoFormatoNumeroInvalido(num);
      }

      if (casasDecimaisDepoisVigula != 0) {
        String cddv = Integer.toString(casasDecimaisDepoisVigula);
        if (sinalExpoente >= 0) {
          if (compareStrings(mantissaExpoente, cddv) > 0) {
            mantissaExpoente = subtracaoNaturalStrings(mantissaExpoente,cddv);
          } else if (compareStrings(mantissaExpoente, cddv) < 0) {
            mantissaExpoente = subtracaoNaturalStrings(cddv,mantissaExpoente);
            sinalExpoente = Number.SINAL_NEGATIVO;
          } else {
             mantissaExpoente = "0";
             sinalExpoente = Number.SINAL_ZERO;
          }
        } else {
            mantissaExpoente = adicaoNaturalStrings(mantissaExpoente,cddv);
        }
      }

      //verificar denominador
      if (!number.equals("") && number.charAt(i) == '/') {
        //trata caso do denominador
        number = number.substring(1,number.length());
        denominador = createNumber(number);
        number = "";
      } else if (!number.equals("")) {
        //deveriam ter acabado os caracteres que formam o numero
         throw new ExcecaoFormatoNumeroInvalido(num);
      }

      n = new Number(mantissa, sinal, sinalExpoente, mantissaExpoente, denominador);
    } else {
      n = new Number(mantissa, sinal, sinalExpoente, mantissaExpoente, denominador);
    }
    return n;
  }

  /**
   * gets the sinal
   * @return
   */
  public int getSinal(){
    return this.sinal;
  }

  /**
   * altera o sinal do numero
   */
  public void setSinal(int sinal){
    if (sinal == SINAL_POSITIVO || sinal == SINAL_NEGATIVO
        || sinal == SINAL_ZERO){
      this.sinal = sinal;
    } else {
      throw new IllegalArgumentException("Parametro invalido:" + sinal);
    }
  }

  /**
   * retorna o denominador
   * @return
   */
  public Number getDenominador() {
    return denominador;
  }

  /**
   * altera o denominador
   * @param denominador
   */
  public void setDenominador(Number denominador) {
    this.denominador = denominador;
  }

  /**
   * retorna a mantissa do expoente
   * @return
   */
  public String getExpoente() {
    return expoente;
  }

  /**
   * altera a mantissa do expoente
   * @param expoente
   */
  public void setExpoente(String expoente) {
    this.expoente = expoente;
  }

  /**
   * retorna a mantissa do expoente
   * @return
   */
  public String getMantissa() {
    return mantissa;
  }

  /**
   * altera a mantissa do expoente
   * @param mantissa
   */
  public void setMantissa(String mantissa) {
    this.mantissa = mantissa;
  }

  /**
   * retorna o sinal do expoente
   * @return
   */
  public int getSinalExpoente() {
     return sinalExpoente;
  }

  /**
   * Altera o sinal do expoente
   * @param sinalExpoente
   */
  public void setSinalExpoente(int sinalExpoente) {
   if (sinalExpoente == SINAL_POSITIVO || sinalExpoente == SINAL_NEGATIVO
       || sinalExpoente == SINAL_ZERO){
     this.sinalExpoente = sinalExpoente;
   } else {
     throw new IllegalArgumentException("Parametro invalido:" + sinalExpoente);
   }
  }

  /****************************************************************************
   *
   * FUNCOES AUXILIARES
   *
   ****************************************************************************/

  /**
   * Completa com zeros a esquerda
   * @param b
   * @return
   */
  private static String completeWithLeftZeros(String number, int numberOfZeros){
    StringBuffer s = new StringBuffer(number);
    s.reverse();
    for(int i=0; i < numberOfZeros; i++){
      try{
        s.append('0');
      } catch(Exception e) {

      }
    }
    s.reverse();
    return s.toString();
  }

  /**
   * Completa com zeros a direita
   * @param b
   * @return
   */
  private static String completeWithRightZeros(String number, int numberOfZeros){
    StringBuffer s = new StringBuffer(number);
    for(int i=0; i < numberOfZeros; i++){
      s.append('0');
    }
    return s.toString();
  }

  /**
   * Completa com zeros a direita
   * @param b
   * @return
   */
  private static String completeWithRightZeros(String number, String numberOfZeros){
    StringBuffer s = new StringBuffer(number);
    while (!numberOfZeros.equals("0")){
        s.append('0');
        numberOfZeros = subtracaoNaturalStrings(numberOfZeros, "1");
    }
    return s.toString();
  }

  /**
   * remove os zeros da esquerda a fim de comparacao
   * @return
   */
  private static String removeLeftZeros(String number){
    int length = number.length();
    if (length > 1) {
      char c = number.charAt(0);
      while(c == '0' && length > 1){
        number = number.substring(1, length);
        c = number.charAt(0);
        length--;
      }
    }
    return number;
  }

  /**
  * remove os zeros a direita a para simplifica as mantissas apos a virgula
  * @return
  */
 private static String removeRightZeros(String number){
   int length = number.length();
   if (length > 1) {
     char c = number.charAt(length-1);
     while(c == '0' && length > 1){
       number = number.substring(0, length-1);
       c = number.charAt(length - 2);
       length--;
     }
   }
   return number;
  }

  /**
   * remove os zeros da esquerda para formatar numero
   * @return
   */
  private static String removeLeftZerosFormatacao(String num){
    String number = num;
    int length = number.length();
    if (length > 1) {
      char c = number.charAt(0);
      while(c == '0' && length > 1){
        number = number.substring(1, length);
        c = number.charAt(0);
        length--;
      }
      if (number.charAt(0) == 'E' || number.charAt(0) == 'e' ||
          number.charAt(0) == '/') {
        number = "0" + number;
      } else if (!number.equalsIgnoreCase(num) &&
                 (number.charAt(0) == '-' || number.charAt(0) == '+')) {
        throw new ExcecaoFormatoNumeroInvalido(num);
      }
    }
    return number;
  }

  /**
   * processa a adicao de dois Strings
   * @param a
   * @param b
   * @return
   */
  private static String adicaoNaturalStrings(String a, String b){
    //resultado da soma
    StringBuffer soma = new StringBuffer("");
    //diferenca entre o tamanho das Strings
    int difTamanho = a.length() - b.length();

    if (difTamanho > 0) {
      b = completeWithLeftZeros(b,difTamanho);
    } else if (difTamanho < 0){
      a = completeWithLeftZeros(a,-difTamanho);
    }
    //valor a ser incluido na proxima soma
    int carry = 0;
    //soma em valores absolutos dos caracteres
    int somaCaracteres = 0;
    //variavel auxiliar para armazenar momentaneamente a soma
    char temp = '0';
    for (int i = a.length()-1; i >= 0; i--) {
      somaCaracteres = carry + a.charAt(i) + b.charAt(i) - 2*CARACTERE_ZERO;
      temp = (char)((somaCaracteres%10) + CARACTERE_ZERO);
      carry = (somaCaracteres/10);
      soma.append(temp);

    }
    if (carry != 0) {
      soma.append(carry);
    }
    soma.reverse();
    return soma.toString();

  }

  /**
   * processa a multiplicacao de dois Strings
   * @param a
   * @param b
   * @return
   */
  private static String multiplicacaoNaturalStrings(String a, String b){
    //produto da multiplicacao
    String produto = "0";
    StringBuffer subProduto = new StringBuffer("");

    b = removeLeftZeros(b);
    a = removeLeftZeros(a);

    //valor a ser incluido na proxima multiplicacao
    int carry = 0;
    //soma em valores absolutos dos caracteres
    int multiplicacaoCaracteres = 0;
    //variavel auxiliar para armazenar momentaneamente o produto
    char temp = '0';
    StringBuffer deslocamento = new StringBuffer("");
    for (int i = b.length()-1; i >= 0; i--) {
      carry = 0;
      subProduto = new StringBuffer(deslocamento.toString());
      for (int j = a.length()-1; j >= 0; j--) {

        multiplicacaoCaracteres = carry + ((a.charAt(j) - CARACTERE_ZERO) * (b.charAt(i)-CARACTERE_ZERO)) ;
        temp = (char)((multiplicacaoCaracteres%10) + CARACTERE_ZERO);
        carry = multiplicacaoCaracteres/10;
        subProduto.append(temp);
      }

      if (carry != 0) {
        subProduto.append(carry);
      }

      subProduto.reverse();

      produto = Number.adicaoNaturalStrings(produto,subProduto.toString());
      deslocamento = deslocamento.append('0');
    }

    return produto;

  }

  /**
   * Metodo mais rapido para verificar se dois strings sao iguais
   * @param a
   * @param b
   * @return
   */
  private static boolean igualdadeStrings (String a, String b) {
    a = Number.removeLeftZeros(a);
    b = Number.removeLeftZeros(b);
    return a.equals(b);
  }


  /**
   * Compara dois Strings numericos
   * @param a
   * @param b
   * @return 1 se a maior que b, -1 se a menor que b e 0 se igual
   */
  private static int compareStrings(String a, String b) {
     int ret  = 0;
     a = removeLeftZeros(a);
     b = removeLeftZeros(b);
     if (a.length() > b.length()) {
       ret = 1;
     } else if (a.length() < b.length()) {
       ret= -1;
     } else {
       ret = a.compareTo(b);
     }
     return ret;
  }


  /**
   * Realiza a subtracao dado 2 Strings a mantissa de "a" deve ser
   * maior que a de "b"
   * @param a
   * @param b
   * @return subtracao entre a e b
   */
  private static String subtracaoNaturalStrings(String a, String b){
    String subtracao = "";
    int dif = a.length() - b.length();

    if (dif > 0) {
      b = completeWithLeftZeros(b, dif);
    } else if (dif < 0) {
      a = completeWithLeftZeros(a, -dif);
    }
    int carry = 0;
    char temp = '0';
    for (int i = a.length()-1; i >= 0; i--) {
      if (a.charAt(i) < b.charAt(i) + carry) {
        temp = (char)(a.charAt(i) + 10);
        temp = (char)(temp - (b.charAt(i) + carry) + CARACTERE_ZERO);
        subtracao = temp + subtracao;
        carry = 1;
      } else {
        temp = (char)(a.charAt(i) - (b.charAt(i) + carry) + CARACTERE_ZERO);
        subtracao = temp + subtracao;
        carry = 0;
      }
    }
    subtracao = Number.removeLeftZeros(subtracao);
    return subtracao;
  }

  /**
   * Retorna o quociente e o resto da divisao natural entre dois Strings
   * @param dividendo
   * @param divisor
   * @return divisao natural
   */
  private static String[] divisaoRestoNaturalStrings(String dividendo, String divisor) throws ExcecaoDivisaoPorZero{
    String quociente = "";
    String resto = "";
    if (igualdadeStrings(divisor,"0")) {
      throw new ExcecaoDivisaoPorZero(dividendo, divisor);
    }
    int comparacao = compareStrings(dividendo, divisor);
    if (comparacao < 0) {
      quociente = "0";
      resto  = dividendo;
    } else if (comparacao == 0){
      quociente = "1";
      resto = "0";
    } else {
      String dividendoInicial = acharSubstringDividendoMaiorQueDivisor(dividendo,divisor);
      String substringDividendo = dividendo.replaceFirst(dividendoInicial,"");
      String[] multiplo = acharMaiorMultiploMenorQueSubstringDividendo(dividendoInicial, divisor);
      quociente = quociente + multiplo[1];
      String diferenca = subtracaoNaturalStrings(dividendoInicial,multiplo[0]);
      while(substringDividendo.length() > 0) {
        diferenca = diferenca + substringDividendo.charAt(0);
        substringDividendo = substringDividendo.substring(1,substringDividendo.length());
        if (compareStrings(diferenca, divisor) < 0) {
          quociente = quociente + "0";
        } else {
          multiplo = acharMaiorMultiploMenorQueSubstringDividendo(diferenca, divisor);
          quociente = quociente + multiplo[1];
          diferenca = subtracaoNaturalStrings(diferenca, multiplo[0]);
        }
      }
      resto = diferenca;
    }
    return new String[] {removeLeftZeros(quociente),removeLeftZeros(resto)};
  }

  /**
   * Acha o menor substring do dividendo maior que o divisor, dividendo deve ser
   * maior ou igual que divisor.
   * @param dividendo
   * @param divisor
   * @return
   */
  private static String acharSubstringDividendoMaiorQueDivisor(String dividendo, String divisor){
    String aux = "";
    boolean cond = true;
    for (int i=0; i <= dividendo.length() && cond; i++) {
      aux = dividendo.substring(0,i);
      if (compareStrings(aux,divisor) > 0) {
        cond = false;
      }
    }
    return aux;
  }

  /**
   * Faz uma serie de multiplicacoes para determinar qual o Maior multiplo
   * do divisor que  menor que o dividendo
   * @param dividendo
   * @param divisor
   * @return posicao 0 valor da multiplicacao, posicao 1 multiplo
   */
  private static String[] acharMaiorMultiploMenorQueSubstringDividendo(
      String subDividendo, String divisor){

    String retorno[] = null;
    String initialNumber ="5";
    String condAnterior = "";
    String produto = "";
    boolean cond = true;
    produto = multiplicacaoNaturalStrings(divisor,initialNumber);
    while (cond) {
      if (compareStrings(produto,subDividendo) > 0) {
        if (condAnterior.equals("menor")) {
          cond = false;
          retorno = new String [] {subtracaoNaturalStrings(produto,divisor),
          subtracaoNaturalStrings(initialNumber,"1")};
        } else {
          condAnterior = "maior";
          initialNumber = subtracaoNaturalStrings(initialNumber,"1");
          produto = subtracaoNaturalStrings(produto,divisor);
        }
      } else if (compareStrings(produto,subDividendo) < 0) {
        if (condAnterior.equalsIgnoreCase("maior")){
          retorno = new String[]{produto,initialNumber};
          cond = false;
        } else {
          condAnterior = "menor";
          initialNumber = adicaoNaturalStrings(initialNumber,"1");
          produto = adicaoNaturalStrings(produto, divisor);
        }
      } else {
        cond = false;
        retorno = new String[]{produto,initialNumber};
      }
    }
    return retorno;
  }

  /**
   * calcula o resto da divisao natural de Strings
   * @param dividendo
   * @param divisor
   * @return
   */
  private static String restoDivisaoNaturalStrings(String dividendo, String divisor) throws ExcecaoDivisaoPorZero{
    return divisaoRestoNaturalStrings(dividendo, divisor)[1];
  }

  /**
   * calcula a o valor da divisao natural de strings
   * @param dividendo
   * @param divisor
   * @return
   */
  private static String divisaoNaturalStrings(String dividendo, String divisor) throws ExcecaoDivisaoPorZero{
    return divisaoRestoNaturalStrings(dividendo, divisor)[0];
  }



  /**
   * Calcula a divisao entre o dividendo e o divisor
   * @param dividendo
   * @param divisor
   * @param fracionar indica se o resultado sera mantido na forma fracionaria
   *                  se este valor for falso ele necessita do parametro
   *                  casasDecimais para indicar a precisao
   * @param casasDecimais Precisao desejada
   * @return Objeto do tipo Number com o resultado da diviso
   * @throws ExcecaoDivisaoPorZero
   */
  private static Number divisaoRacionalStrings(String dividendo, String divisor,
      boolean fracionar, int casasDecimais) throws ExcecaoDivisaoPorZero {

    String mdc = mdc(dividendo,divisor);

    if (!mdc.equals("1")) {
      dividendo = divisaoNaturalStrings(dividendo,mdc);
      divisor = divisaoNaturalStrings(divisor,mdc);
    }
    String [] divisaoInteira = divisaoRestoNaturalStrings(dividendo, divisor);
   /*
       ht guarda a tupla (resto, caractere), ou seja,
       os restos parciais e os indices dos caracteres multiplicadores
       deste resto
    */
    Hashtable ht = new Hashtable();
    divisaoInteira[1] = removeLeftZeros(divisaoInteira[1]);
    Number resultado = new Number();

    if (divisaoInteira[1].equalsIgnoreCase("0")) {
      resultado.sinal = SINAL_POSITIVO;
      resultado.mantissa = divisaoInteira[0];
    } else {
      //guarda o quociente da diviso calculado apos a virgula
      String quocienteDepoisVirgula = "";
      //indica em que indice da string quocienteDepoisVirgula a dizima comeca
      int inicioDizima = 0;
      String resto = divisaoInteira[1];
      String multiplo[] = new String[2];
      //indica se algum resto parcial ainda no se repetiu
      boolean cond = true;
      String quocienteAvaliado = "";
      boolean isparteInteiraZero = divisaoInteira[0].equalsIgnoreCase("0");
      while (cond && !removeLeftZeros(resto).equalsIgnoreCase("0")
             &&(fracionar || quocienteAvaliado.length() < casasDecimais + 1)){
        resto = resto + "0";
        while (compareStrings(resto, divisor) < 0 ) {
          resto = resto + "0";
          quocienteDepoisVirgula = quocienteDepoisVirgula + "0";
        }
        if (!ht.containsKey(resto)) {
           //hs.add(resto);
           ht.put(resto, "" + quocienteDepoisVirgula.length());
           multiplo = acharMaiorMultiploMenorQueSubstringDividendo(resto, divisor);
           quocienteDepoisVirgula = quocienteDepoisVirgula + multiplo[1];
           resto = subtracaoNaturalStrings(resto,multiplo[0]);

        } else {
          cond = false;
          inicioDizima = Integer.parseInt((String)ht.get(resto));
        }
        quocienteAvaliado = removeLeftZeros(quocienteDepoisVirgula);
      }

      if (removeLeftZeros(resto).equalsIgnoreCase("0")){

       /*resultado = divisaoInteira[0] + quocienteDepoisVirgula
                 + "E-" + (quocienteDepoisVirgula.length() + incrementoExpoente);*/

       resultado = new Number(
           divisaoInteira[0] + quocienteDepoisVirgula,
           SINAL_POSITIVO,
           SINAL_NEGATIVO,
           "" + quocienteDepoisVirgula.length(),
           null);

       /*resultado.sinal = SINAL_POSITIVO;
       resultado.mantissa = divisaoInteira[0] + quocienteDepoisVirgula;
       resultado.sinalExpoente = SINAL_NEGATIVO;
       resultado.expoente = "" + quocienteDepoisVirgula.length();*/

      } else {
        //caracteres periodicos da dizima
        String dizima = quocienteDepoisVirgula.substring(inicioDizima,
            quocienteDepoisVirgula.length());

        if (fracionar) {

           String mantissaDividendo = subtracaoNaturalStrings(
                   divisaoInteira[0] + quocienteDepoisVirgula + dizima,
                   divisaoInteira[0] + quocienteDepoisVirgula);

           String [] calcularDizima = calculaZerosDenominadorFracaoGeratriz(mantissaDividendo,
               (quocienteDepoisVirgula).length());
           mantissaDividendo = calcularDizima[0];

           String denominadorFracao = subtracaoNaturalStrings(completeWithRightZeros("1",
               dizima.length()),"1");
           denominadorFracao = denominadorFracao + calcularDizima[1];
           if (compareStrings(denominadorFracao,divisor) < 0) {
             /*resultado =  subtracaoNaturalStrings(divisaoInteira[0] + quocienteDepoisVirgula,
                 divisaoInteira[0]) +  "/" + denominadorFracao;*/
               resultado.sinal = SINAL_POSITIVO;
               resultado.mantissa = mantissaDividendo;
               resultado.denominador = new Number();
               resultado.denominador.sinal = SINAL_POSITIVO;
               resultado.denominador.mantissa = denominadorFracao;

           } else {
               //resultado = dividendo + "/" + divisor;

               resultado.sinal = SINAL_POSITIVO;
               resultado.mantissa = dividendo;
               resultado.denominador = new Number();
               resultado.denominador.sinal = SINAL_POSITIVO;
               resultado.denominador.mantissa = divisor;
           }
        } else {
           //String dizima = quocienteDepoisVirgula.substring(inicioDizima,quocienteDepoisVirgula.length());

           if (casasDecimais <= quocienteDepoisVirgula.length()) {

             /*resultado = divisaoInteira[0] + quocienteDepoisVirgula.substring(0,casasDecimais)
                       + "E-" + (casasDecimais + incrementoExpoente + numerosZerosAntesDizima);*/
             String arredondamento = null;
             if (!isparteInteiraZero){
               arredondamento = arredondar(divisaoInteira[0] + quocienteDepoisVirgula + dizima.charAt(0),
                            casasDecimais + divisaoInteira[0].length());
             } else {
               arredondamento = arredondar(quocienteAvaliado, casasDecimais);
             }

             resultado = new Number(
                 arredondamento,
                 SINAL_POSITIVO,
                 SINAL_NEGATIVO,(casasDecimais +
                   (quocienteDepoisVirgula.length() - quocienteAvaliado.length())) + "",
                 null);

             /*resultado.sinal = SINAL_POSITIVO;
             resultado.mantissa = divisaoInteira[0] + arredondar(quocienteDepoisVirgula,casasDecimais);
             resultado.sinalExpoente = SINAL_NEGATIVO;
             resultado.expoente = "" + quocienteDepoisVirgula.substring(0,casasDecimais).length();*/
             //(casasDecimais - incrementoExpoente + numerosZerosAntesDizima);


           } else {
              int numeroCaracteresAntesDizima = quocienteDepoisVirgula.length() - dizima.length();
              int repeticoesDizima = (casasDecimais - numeroCaracteresAntesDizima)/dizima.length();
              int tamanhoSubStringDizima = (casasDecimais - numeroCaracteresAntesDizima)%dizima.length();
              String resultadoString = quocienteDepoisVirgula.replaceFirst(dizima,"");
              for (int i = 0; i < repeticoesDizima; i++) {
                 resultadoString = resultadoString + dizima;
              }
              resultadoString = resultadoString + dizima.substring(0,tamanhoSubStringDizima + 1);
              resultadoString = arredondar(resultadoString, casasDecimais);
              resultadoString = divisaoInteira[0] + resultadoString;
              /*
              resultado = resultadoString + dizima.substring(0,tamanhoSubStringDizima)
                        + "E-" + (numerosZerosAntesDizima + tamanhoSubStringDizima
                        + (repeticoesDizima*dizima.length()) + incrementoExpoente);*/

             resultado = new Number(
                 resultadoString,
                 SINAL_POSITIVO,
                 SINAL_NEGATIVO,
                 "" + casasDecimais,
                 null);

              /*resultado.sinal = SINAL_POSITIVO;
              resultado.mantissa = resultadoString;
              resultado.sinalExpoente = SINAL_NEGATIVO;
              resultado.expoente = "" + (numerosZerosAntesDizima + tamanhoSubStringDizima
                         + (repeticoesDizima*dizima.length()));*/
           }
        }

      }
    }
    resultado.mantissa = removeLeftZeros(resultado.mantissa);
    return resultado;

  }


  /**
   * Calcula a quantidade de zeros que serao concatenados ao denominador
   * fracionario.
   * @param mantissa caracteres do dividendo
   * @param casasDecimaisDepoisVirgula indica a quandidade de casas
   * decimais depois da virgula
   * @return  posicao 0 mantissa sem os zeros a direita
   *          posicao 1 String com zeros a serem concatenados ao divisor
   */
  private static String [] calculaZerosDenominadorFracaoGeratriz(String mantissa,
      int casasDecimaisDepoisVirgula){

    String [] resultado = new String[2];
    int len = mantissa.length();
    int i = len-1;
    int numZeros;
    String zeros = "";
    // se o resultado da subtracao deu uma cadeia de caracteres
    // menor que a quantidade de casas decimais, devido a remoao
    // dos zeros a esquerda
    if (len < casasDecimaisDepoisVirgula) {
      //sai do lao quando encontramos o primeiro caractere diferente de zero.
      while (mantissa.charAt(i) == '0' && i >= 0) {
        i--;
      }
    } else {
      while (mantissa.charAt(i) == '0' && i >= (len - casasDecimaisDepoisVirgula)) {
        i--;
      }
    }
    numZeros = casasDecimaisDepoisVirgula - (len-(i+1));
    zeros = completeWithRightZeros(zeros,numZeros);

    resultado[0] = mantissa.substring(0,i+1);
    resultado[1] = zeros;
    return resultado;
  }

  /**
   * Arredonda para o mais proximo
   * @param numero cadeia de caracteres
   * @param precisao
   * @return
   */
  private static String arredondar (String numero, int precisao) {

    if (numero.length() > precisao) {
      if (numero.charAt(precisao) >= '5' ) {
        numero = adicaoNaturalStrings(numero.substring(0,precisao), "1");
      } else {
        numero = numero.substring(0,precisao);
      }
      //numero = numero.substring(0, precisao);
    }
    return numero;
  }

  /**
   * Imprime o String numerico
   * @return um string no formato do IEEE
   */
  public String toString(){
    String r = "";
    switch(this.sinal){
      case Number.SINAL_NEGATIVO : r = "-"; break;
      default: break;
    }
    r = r + this.mantissa;
    if (!this.expoente.equals("0")){
      r = r + "E";
      switch(this.sinalExpoente) {
        case Number.SINAL_NEGATIVO : r = r + "-"; break;
                            default: break;
      }
      r = r + this.expoente;
    }
    if (this.denominador != null) {
      r = r + "/(" + this.denominador.toString() + ")";
    }
    return r;
  }


  /**
   * Verifica se existe divisao por zero neste divisor
   * @param n
   */
  private boolean verificaExcecaoDivisaoPorZero(Number n) throws ExcecaoDivisaoPorZero {
    boolean excecaoDivisaoPorZero = false;
    if (n.denominador != null) {
      if (n.denominador.mantissa.equals("0")){
        throw new ExcecaoDivisaoPorZero(n.denominador.toString());
      } else {
        excecaoDivisaoPorZero = excecaoDivisaoPorZero || verificaExcecaoDivisaoPorZero(n.denominador);
      }
    }
    if (n.mantissa.equals("0")){
       throw new ExcecaoDivisaoPorZero(n.toString());
    }
    return excecaoDivisaoPorZero;
  }

  /**
   * Faz o parsing do expoente
   * @param number
   * @return na posicao 0 o String processado,
   *         na posicao 1 a mantissa do expoente e
   *         na posicao 2 o sinal do expoente
   */
  private static String [] parseExpoente(String number, String mantissaExpoente, int sinalExpoente) {
     String [] ret = new String[3];
     int i = 0;
     number = number.substring(1,number.length());
     //verifica sinal do expoente
     //sinal positivo
     if (number.charAt(i)=='+' ||
       number.charAt(0) >= CARACTERE_ZERO && number.charAt(0) <='9'){
       sinalExpoente = Number.SINAL_POSITIVO;
       //sinal negativo
     } else if (number.charAt(i)=='-') {
        sinalExpoente = Number.SINAL_NEGATIVO;
     }
     if (number.charAt(i)=='+' ||
         number.charAt(i)=='-') {
         number = number.substring(1,number.length());
     }
     i=0;
     while ( i < number.length() && number.charAt(i) >= CARACTERE_ZERO && number.charAt(i) <= '9') {
       i++;
     }

     if (i < number.length() && number.charAt(i) != '/') {
       throw new ExcecaoFormatoNumeroInvalido("");
     }
     mantissaExpoente = number.substring(0,i);
     number = number.substring(i,number.length());
     ret[0] = number;
     ret[1] = mantissaExpoente;
     ret[2] = Integer.toString(sinalExpoente);
     return ret;
  }

  /**
   * Calcula o minino multiplo comum de 2 numeros
   * @param a String numerico
   * @param b String numerico
   * @return mmc de a e b
   * @throws ExcecaoDivisaoPorZero
   */
  public static String mmc(String a, String b) throws ExcecaoDivisaoPorZero{
    return divisaoNaturalStrings(multiplicacaoNaturalStrings(a,b), mdc(a,b));
  }

  /**
   * calcula o maximo divisor comum de e 2 numeros
   * @param m
   * @param n
   * @return
   * @throws ExcecaoDivisaoPorZero
   */
  public static String mdc(String m, String n){
    String mdc = "1";
    if (m.equals("1") || n.equals("1")) {
      mdc = "1";
    } else {
      mdc = mdcRecursivo(m,n);
    }
    return mdc;
  }

  /**
   * calcula o maximo divisor comum de e 2 numeros
   * @param m
   * @param n
   * @return
   * @throws ExcecaoDivisaoPorZero
   */
  private static String mdcRecursivo(String m, String n) throws ExcecaoDivisaoPorZero{
    String mdc = "";

      if (m.equals("0")){
          mdc = n;
      } else {
         if (compareStrings(n,m) >=0) {
            mdc = mdcRecursivo(restoDivisaoNaturalStrings(n,m),m);
         } else {
            mdc = mdcRecursivo(restoDivisaoNaturalStrings(m,n),n);
         }
      }

    return mdc;
  }



 /*************************************************************************
  *  FUNCOES REAIS
  *****************************************************************************/

 /**
   * compara dois Objetos Number, se a for maior que b retorna um numero
   * @param a
   * @param b
   * @return 1 se a for maior que b,
   *         0 se a for igual a b
   *      e -1 se a for menor que b
   */
  public static int comparar (Number a, Number b) {
    int ret = 0;
    Number m = a.clonar();
    Number n = b.clonar();
    if (m.sinal > n.sinal){
      ret = 1;
    } else if (m.sinal < n.sinal) {
      ret = -1;
    } else { //sinais iguais

      colocarMesmaBaseDenominadores(m,n);
      m.denominador = null;
      n.denominador = null;
      normalizarNumerosDesconsiderandoDenominador(m,n);
      ret = compareStrings(m.mantissa,n.mantissa);

      if (m.sinal == SINAL_NEGATIVO) {
        ret = -ret;
      }

    }

    return ret;
  }

  /**
   * calcula o maximo entre dois numeros
   * @param a
   * @param b
   * @return
   */
  public static Number max (Number a, Number b){
    Number n = null;
    if (Number.comparar(a,b) >= 0) {
      n = a;
    } else {
      n = b;
    }
    return n;
  }

  /**
   * Calcula o minimo entre dois numeros
   * @param a
   * @param b
   * @return
   */
  public static Number min (Number a, Number b){
    Number n = null;
      if (Number.comparar(a,b) <= 0) {
        n = a;
      } else {
        n = b;
      }
    return n;
  }

  /**
   * retorna o modulo de um numero
   * @param a
   * @return
   */
  public Number modulo (){
   Number r = this.clonar();
   if (r.sinal == SINAL_NEGATIVO) {
     r.sinal = SINAL_POSITIVO;
   }
   return r;
  }

  /**
   * retorna o modulo de um numero
   * @param a
   * @return
   */
  public static Number modulo (Number a){
   Number r = a.clonar();
   if (r.sinal == SINAL_NEGATIVO) {
     r.sinal = SINAL_POSITIVO;
   }
   return r;
  }

  /**
   * Adicao real de Strings
   * @param a
   * @param b
   * @return
   */
  public static Number adicionar(Number a, Number b){
    return adicaoOuSubtracaoRacional(a,b,true);
  }

  /**
   * realiza a adicao ou subtracao dependendo do parametro isAdicao
   * @param a uma parcela ou minuendo
   * @param b uma parcela ou subtraendo
   * @param isSoma se true indica soma caso false indica subtracao.
   * @return
   */
  private static Number adicaoOuSubtracaoRacional(Number a, Number b, boolean isAdicao){
     Number r  = new Number();
     colocarMesmaBaseDenominadores(a,b);
     r = adicaoOuSubtracaoSemDenominadores(a,b,isAdicao);
     if (a.denominador != null) {
        r.denominador = a.denominador.clonar();
     }
     return r;
  }

  /**
   * Subtracao real de Strings
   * @param a
   * @param b
   * @return
   */
  public static Number subtrair(Number a, Number b){
     return adicaoOuSubtracaoRacional(a,b,false);
  }

  /**
   * Multiplica dois numeros a e b
   * @param a
   * @param b
   * @return o resultado da multiplicao entre A e B
   */
  public static Number multiplicar (Number a, Number b) {

    Number r = multiplicacaoDesconsiderandoDenominador(a,b);

    if (a.denominador == null && b.denominador != null) {
      r.denominador = b.denominador.clonar();
    } else if (a.denominador != null && b.denominador == null) {
      r.denominador = a.denominador.clonar();
    } else if (a.denominador != null && b.denominador != null) {
      r.denominador = multiplicacaoDesconsiderandoDenominador(a.denominador, b.denominador);
    }
    return r;
  }

  /**
   * Multiplica dois numeros a e b
   * @param a
   * @param b
   * @return o resultado da multiplicao entre A e B
   */
  public static Number dividir (Number a, Number b, boolean fracionar, int casasDecimais)
      throws ExcecaoDivisaoPorZero {

    if (a.denominador == null) {
      try {
        a.denominador = new Number("1");
      } catch (Exception e) {

      }
    }
    if (b.denominador == null) {
      try {
        b.denominador = new Number("1");
      } catch (Exception e) {

      }
    }

    Number r = multiplicacaoDesconsiderandoDenominador(a,b.denominador);
    r.denominador = multiplicacaoDesconsiderandoDenominador(a.denominador,b);
    //se tivermos que dividir o numero;
    if (!fracionar) {
      eliminaExpoenteSinalDenominador(r);
      Number divisaoMantissas = divisaoRacionalStrings(r.mantissa,r.denominador.mantissa,
          false, casasDecimais);

      r.mantissa = divisaoMantissas.mantissa;
      String [] expoente = avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(r.expoente,
          divisaoMantissas.expoente, r.sinalExpoente, divisaoMantissas.sinalExpoente);
      r.sinalExpoente = Integer.parseInt(expoente[0]);
      r.expoente = expoente[1];
      r.denominador = null;

    } else {
        eliminaExpoenteSinalDenominador(r);
        simplificarMantissaFracao(r);
    }
    return r;
  }

  /**
   * calcula a Adicao ou subtracao entre dois numeros ignorando os divisores dos mesmos
   * @param a um objeto numero
   * @param b um objeto numero
   * @param isAdicao define se uma Adicao ou subtracao ser efetuada
   * @return a soma de dois numeros sem divisores
   */
  private static Number adicaoOuSubtracaoSemDenominadores(Number a, Number b, boolean isAdicao){
    Number n = new Number();
    if (!isAdicao) {
      b.sinal = -b.sinal;
    }
    normalizarNumerosDesconsiderandoDenominador(a,b);

    String [] soma = avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(a.mantissa,
        b.mantissa, a.sinal, b.sinal);

    n.sinalExpoente = a.sinalExpoente;
    n.expoente = a.expoente;
    n.sinal = Integer.parseInt(soma[0]);
    n.mantissa = soma[1];
    if (!isAdicao) {
      b.sinal = -b.sinal;
    }

    if (n.mantissa.equalsIgnoreCase("0")){
      n.expoente = "0";
      n.setSinalExpoente(SINAL_ZERO);
    }
    return n;
  }


  /**
   * Avalia os sinais e tamanhos das mantissas e realiza as operaoes de Adicao
   * ou subtracao
   *
   * @param mantissaA  mantissa do numero A
   * @param mantissaB mantissa do numero B
   * @param sinalA sinal do numero A
   * @param sinalB sinal do numero B
   *
   * @return um array de string de 2 posicoes, na primeira posicao do array esta
   *         o sinal codificado como -1(sinal negativo), 0 (sinal neutro) e
   *         1 (sinal positivo), na segunda posicao o resultado da mantissa da
   *         operacao
   *
   */
  private static String[] avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(String mantissaA,
      String mantissaB, int sinalA, int sinalB) {

    String result [] = new String[2];
    if (sinalA == sinalB) {
      result[0] = sinalA + "";
      result[1] = adicaoNaturalStrings(mantissaA, mantissaB);
    } else {
      if (compareStrings(mantissaA, mantissaB) > 0) {
        result[0] = sinalA + "";
        result[1] = subtracaoNaturalStrings(mantissaA, mantissaB);
      } else if (compareStrings(mantissaA, mantissaB) < 0){
        result[0] = sinalB + "";
        result[1] = subtracaoNaturalStrings(mantissaB, mantissaA);
      } else {
        result[0] = SINAL_ZERO + "";
        result[1] = "0";
      }
    }
    return result;
  }

  /**
   * Multiplica dois numeros que no possuem divisores
   * @param a parcela a
   * @param b parcela b
   * @return multiplicacao entre a e b
   */
  private static Number multiplicacaoDesconsiderandoDenominador(Number a, Number b){
    Number n = new Number();
    n.sinal = a.sinal * b.sinal;
    String expoente [] = avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(a.expoente,
        b.expoente, a.sinalExpoente, b.sinalExpoente);
    n.sinalExpoente = Integer.parseInt(expoente[0]);
    n.expoente = expoente[1];
    n.mantissa = multiplicacaoNaturalStrings(a.mantissa, b.mantissa);
    return n;
  }

  /**
   * Coloca os numeros a e b sob o mesmo expoente
   * @param a
   * @param b
   */
  private static void normalizarNumerosDesconsiderandoDenominador(Number a, Number b) {
      boolean aMaior = false;
      //mesmos sinais
    if (a.sinalExpoente == b.sinalExpoente) {
       String diferencaExpoentes = "0";
      if ((compareStrings(a.expoente, b.expoente) > 0)) {
        diferencaExpoentes = subtracaoNaturalStrings(a.expoente, b.expoente);
        aMaior = true;
      } else if (compareStrings(a.expoente, b.expoente) < 0){
        diferencaExpoentes = subtracaoNaturalStrings(b.expoente, a.expoente);
      }

      if ((a.sinalExpoente == SINAL_POSITIVO && aMaior) ||
          (a.sinalExpoente == SINAL_NEGATIVO && !aMaior)) {
          a.expoente = b.expoente;
          a.mantissa = completeWithRightZeros(a.mantissa,diferencaExpoentes);
      } else {
          b.expoente = a.expoente;
          b.mantissa = completeWithRightZeros(b.mantissa,diferencaExpoentes);
      }
    } else {
      //sinais diferentes
      String somaExpoentes = "";
      somaExpoentes = adicaoNaturalStrings(a.expoente,b.expoente);
      if (a.sinalExpoente > b.sinalExpoente) {
        a.sinalExpoente = b.sinalExpoente;
        a.expoente = b.expoente;
        a.mantissa = completeWithRightZeros(a.mantissa, somaExpoentes);
      } else if (b.sinalExpoente > a.sinalExpoente){
        b.sinalExpoente = a.sinalExpoente;
        b.expoente = a.expoente;
        b.mantissa = completeWithRightZeros(b.mantissa, somaExpoentes);
      }
    }
  }

  /**
   * Coloca na mesma base de divisores
   * @param a numero a ser colocado numa base comum que b
   * @param b numero a ser colocado numa base comum que a
   */
  private static void colocarMesmaBaseDenominadores(Number a, Number b){

    if (a.denominador != null && b.denominador == null) {
      ajustarParaNovaBase(a,b);
    } else if (a.denominador == null && b.denominador != null) {
      ajustarParaNovaBase(b,a);
    } else if (a.denominador != null && b.denominador != null){

      //elimina sinal e potencia dos denominadores
      eliminaExpoenteSinalDenominador(a);
      eliminaExpoenteSinalDenominador(b);
      //calcula mmc
      String mmc = mmc(a.denominador.mantissa, b.denominador.mantissa);

      //calcula o resultado da divisao do mmc pelo denominador atual
      String fatorMultiplicativoA = divisaoNaturalStrings(mmc,a.denominador.mantissa);
      String fatorMultiplicativoB = divisaoNaturalStrings(mmc,b.denominador.mantissa);

      //atribui aos denominadores o mmc
      a.denominador.mantissa = mmc;
      b.denominador.mantissa = mmc;

      //calcula o dividendo
      a.mantissa = multiplicacaoNaturalStrings(fatorMultiplicativoA,a.mantissa);
      b.mantissa = multiplicacaoNaturalStrings(fatorMultiplicativoB,b.mantissa);
    }

  }


  /**
   * Elimina o sinal do Denominador e o seu expoente para calcular o mmc das mantissas
   * @param m
   * @throws NumberFormatException
   */
  private static void eliminaExpoenteSinalDenominador(Number m) {
    String [] expoenteA = avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(m.expoente,
        m.denominador.expoente, m.sinalExpoente, -m.denominador.sinalExpoente);
    m.denominador.expoente = "0";
    m.denominador.sinalExpoente = SINAL_ZERO;
    m.sinalExpoente = Integer.parseInt(expoenteA[0]);
    m.expoente = expoenteA[1];
    if (m.denominador.sinal == SINAL_NEGATIVO){
      m.sinal = -m.sinal;
      m.denominador.sinal = SINAL_POSITIVO;
    }
  }

  /**
   * Ajusta o numero para nova base
   * @param a numero sem Denominador
   * @param b numero com Denominador
   */
  private static void ajustarParaNovaBase(Number m, Number n) {
    Number produto = multiplicacaoDesconsiderandoDenominador(n, m.denominador);
    n.sinal = produto.sinal * m.denominador.sinal;
    n.mantissa = produto.mantissa;
    n.expoente = produto.expoente;
    n.sinalExpoente = produto.sinalExpoente;
    n.denominador = m.denominador.clonar();
  }


  /**
   * Funcao recursiva para simplifica cadeia de denominadores
   * @param a
   */
  private static void simplificarDenominadores (Number a) {
    Number c;
    if (a.denominador != null && a.denominador.denominador != null){
       simplificarDenominadores(a.denominador);
       c = multiplicacaoDesconsiderandoDenominador(a,a.denominador.denominador);
       a.mantissa  = c.mantissa;
       a.sinal = c.sinal;
       a.sinalExpoente = c.sinalExpoente;
       a.expoente = c.expoente;
       a.denominador.denominador = null;
    }
  }

  /**
   * calcula o mdc e reduz a fracao
   * @param a
   */
  private static void simplificarMantissaFracao(Number a){
    if (a.denominador != null){
      String mdc = mdc(a.mantissa, a.denominador.mantissa);
      if (!mdc.equals("1")) {
        a.denominador.mantissa = divisaoNaturalStrings(a.denominador.mantissa,mdc);
        a.mantissa = divisaoNaturalStrings(a.mantissa,mdc);
      }
    }
  }

  /**
   * Calcula o inverso multiplicativo o mesmo que elevar a -1
   * @param a
   * @return
   */
  public static Number inversoMultiplicativo(Number a) throws ExcecaoDivisaoPorZero{
     Number inv= new Number();
     if (a != null ){
       if (!a.mantissa.equalsIgnoreCase("0")){
         if (a.getDenominador() != null) {
           inv.setSinal(a.getDenominador().getSinal());
           inv.setMantissa(a.getDenominador().getMantissa());
           inv.setSinalExpoente(a.getDenominador().getSinalExpoente());
           inv.setExpoente(a.getDenominador().getExpoente());
         } else {
           inv.setSinal(Number.SINAL_POSITIVO);
           inv.setMantissa("1");
         }
         inv.setDenominador(new Number());
         inv.getDenominador().setSinal(a.getSinal());
         inv.getDenominador().setMantissa(a.getMantissa());
         inv.getDenominador().setExpoente(a.getExpoente());
         inv.getDenominador().setSinalExpoente(a.getSinalExpoente());
         eliminaExpoenteSinalDenominador(inv);
         return inv;
       } else {
         throw new ExcecaoDivisaoPorZero("Impossivel inverter o numero Zero:" + a);
       }
    } else {
      throw new IllegalArgumentException("Number inversoMultiplicativo nao pode ser nulo");
    }
  }

  /**
   * Cria uma instancia identica ao objeto
   * @returns um objeto identico com outro endereco de memoria
   */
  public Number clonar(){
    Number m = new Number();
    m.sinal = this.sinal;
    m.mantissa = this.mantissa;
    m.sinalExpoente = this.sinalExpoente;
    m.expoente = this.expoente;
    if (this.denominador != null) {
      m.denominador = this.denominador.clonar();
    }
    return m;
  }

  /**
   * Prints the Number in Scientific Notation
   * @param precision
   * @return
   */
  public String toScientificNotation(int precision){
    String ret = "";
    Number n = this.clonar();

    //resolves fraction
    if (this.denominador != null) {
      n.denominador = null;
      n = Number.dividir(n, this.denominador, false, precision + 1);
    }

    String expoente [] = this.avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(
    n.expoente, (n.mantissa.length() - 1) + "" , n.sinalExpoente, SINAL_POSITIVO);
    n.expoente = expoente[1];
    n.sinalExpoente = Integer.parseInt(expoente[0]);

    //round
    if (n.mantissa.length() > precision + 1) {
      n.mantissa = this.arredondar(n.mantissa, precision + 1);
    }

    //concatenates signal
    if (n.sinal == SINAL_NEGATIVO){
      ret = ret + "-";
    }
    n.mantissa  = this.removeRightZeros(n.mantissa);

    //concatenates mantissa
    if (n.mantissa.length() > 1){
      ret = ret + n.mantissa.substring(0,1) + "," + n.mantissa.substring(1,n.mantissa.length());
    } else {
      ret = ret + n.mantissa;
    }

    //concatenates power
    if (!n.mantissa.equalsIgnoreCase("0") && !n.expoente.equalsIgnoreCase("0")) {
      ret = ret + "E";
      if (n.sinalExpoente == SINAL_NEGATIVO){
       ret = ret + "-";
      }
      ret = ret + n.expoente;
    }

    return ret;
  }

  /**
   * testa a operao de adio
   */
  private static void testeAdicao(){
    String a = "9857433243498432837248972948792837424";
    String b = "932480923482343248923897492734987324879832";
    System.out.println("adicao           : " + Number.adicaoNaturalStrings(a,b));

    double ad = 9857433243498432837248972948792837424.;
    double bd = 932480923482343248923897492734987324879832.;
    System.out.println("adicao double    : " +  (ad + bd));

    BigDecimal ab = new BigDecimal(9857433243498432837248972948792837424.);
    BigDecimal bb = new BigDecimal(932480923482343248923897492734987324879832.);
    System.out.println("adicao BigDecimal: " +  (ab.add(bb)));
  }

  /**
   * testa a operao de subtracao
   */
  private static void testeSubtracao(){
    String a = "10000000000000000000000000004343494";
    String b = "10000000000000234343443430000000000000";
    System.out.println("subtracao           : " + Number.subtracaoNaturalStrings(b,a));

    double ad = 10000000000000000000000000004343494.;
    double bd = 10000000000000234343443430000000000000.;
    System.out.println("subtracao double    : " +  (bd - ad));

    BigDecimal ab = new BigDecimal(10000000000000000000000000004343494.);
    BigDecimal bb = new BigDecimal(10000000000000234343443430000000000000.);
    System.out.println("subtracao BigDecimal: " +  (bb.subtract(ab)));
  }

  /**
   * testa a operao de subtracao
   */
  private static void testeMultiplicacao(){
    String a = "9864959123598765767463";
    String b = "19875897459485";
    System.out.println("mult           : " + Number.multiplicacaoNaturalStrings(b,a));

    double ad = 9864959123598765767463.;
    double bd = 19875897459485.;
    System.out.println("mult double    : " +  (bd * ad));

    BigDecimal ab = new BigDecimal(9864959123598765767463.);
    BigDecimal bb = new BigDecimal(19875897459485.);
    System.out.println("mult BigDecimal: " +  (bb.multiply(ab)));
  }

  /**
   * teste divisao Inteira
   */
  private static void testaDivisaoInteira () {
    String a = "97435987459375743";
    String b = "3427598782398423894983248973284324324";
    System.out.println("div: " + Number.divisaoNaturalStrings(b,a));
  }

  private static void testaRestoDivisaoInteira () {
    String a = "97435987459375743";
    String b = "3427598782398423894983248973284324324";
    System.out.println("resto: " + Number.restoDivisaoNaturalStrings(b,a));
  }

  private static void testeMDC() {
    String a = "8726346348765843765";
    String b = "982734873297492837432";
    System.out.println("mdc: " + Number.mdc(b,a));
  }

  private static void testeMMC() {
    String a = "50400";
    String b = "1230390";
    System.out.println("mmc: " + Number.mmc(b,a));
  }

  /**
   * Metodo para testes
   * @param args
   */
  public static void main(String args[]){
    //Number.testeMDC();
    //Number.testeMMC();

  }


}

/*
    java.math.BigDecimal f = new java.math.BigDecimal(10E29);
    java.math.BigDecimal g = new java.math.BigDecimal(1);
    java.math.BigDecimal h = new java.math.BigDecimal(10E28);
    f = f.add(g);
    System.out.println(f);
    f = f.subtract(h);
    System.out.println(f);
    double d = 10e20 + 1 - 10e20;
    System.out.println(d);
    System.out.println(Number.adicaoNaturalStrings("36", "15"));
    System.out.println(Number.completeWithLeftZeros("1234",5));
    System.out.println(Number.removeLeftZeros("0000000123"));

    Number.igualdadeStrings("00000001230","01230");
    String s= "";
    long time1 = 0;

    time1 = System.currentTimeMillis();
    for (int i = 0; i < 100000; i ++) {
      s = Number.adicaoNaturalStrings("3600000000000000000001", "1500000000000000009");
    }
    time1 = System.currentTimeMillis() - time1;
    System.out.println("tempo da operacao c/ Strings= " + time1);
    System.out.println("resultado c/ Strings= " + s);

    time1 = System.currentTimeMillis();
    for (int i = 0; i < 100000; i ++) {
      d = 3600000000000000000001. + 1500000000000000009.;
    }
    time1 = System.currentTimeMillis() - time1;
    System.out.println("tempo da operacao c/ double= " + time1);
    System.out.println("resultado c/ double= " + d);

    Number a = new Number("10",SINAL_POSITIVO);
    Number b = new Number("10",SINAL_NEGATIVO);
    System.out.println("comparacao:" + Number.comparar(a,b));
    System.out.println("subtracao:" + Number.subtracaoNaturalStrings(Number.adicaoNaturalStrings("100000000000000000000000000000","1"),"10000000000000000000000000000"));

    time1 = System.currentTimeMillis();
    for (int i = 1; i <= 100000; i ++) {
      s = Number.multiplicacaoNaturalStrings("4010010010101", "991111199999");
    }
    time1 = System.currentTimeMillis() - time1;
    System.out.println("tempo da * operacao c/ String= " + time1);
    System.out.println("resultado c/ String= " + s);

    time1 = System.currentTimeMillis();
    for (int i = 1; i <= 100000; i ++) {
      d = 4010010010101. * 991111199999.;
    }
    time1 = System.currentTimeMillis() - time1;
    System.out.println("tempo da operacao *  c/ double= " + time1);
    System.out.println("resultado * c/ double= " + d);

    java.math.BigDecimal k = new java.math.BigDecimal("988888888888888888888888888887988888888888888888888888888887");
    //double k = 988888888888888888888888888887.;
    java.math.BigDecimal i = new java.math.BigDecimal("0");
    //double i = 0;
    java.math.BigDecimal r = new java.math.BigDecimal("7888823");

    //double r = 7888823;

    long time1 = System.currentTimeMillis();
    //time1 = System.currentTimeMillis();
    for (int j = 0 ; j < 1000; j++) {
      i = k.divide(r,java.math.BigDecimal.ROUND_UP);
    }
    time1 = System.currentTimeMillis() - time1;

    System.out.println("Tempo Divisao BigDecimal:" + time1);

    time1 = System.currentTimeMillis();
    String [] div = new String[2];
    for (int j = 0 ; j < 1000; j++) {
      div = Number.divisaoRestoNaturalStrings("988888888888888888888888888887988888888888888888888888888887","7888823");
    }
    time1 = System.currentTimeMillis() - time1;
    System.out.println("    Divisao BigDecimal:" + i);
    System.out.println("       Divisao Strings:" + div[0] + " resto: " + div[1]);
    System.out.println(" Tempo Divisao Strings:" + time1);
    */

    //try {
      //Number n = createNumber("44,234E-11/00010101,93934E123/1,43546E+1200000/1/12E-1/1,34E29/9");
      //Number n = createNumber("-1,000E23/23,3434/2");
      //System.out.println("Numero: " + n.toString());
      //System.out.println(Number.divisaoRealStrings("105","303", true, 5));
      //System.out.println(mmc("100","3025"));
        //Number a = Number.createNumber("1000E13/1");
        //Number b = Number.createNumber("-9192912/1E2");
      //Number [] s = Number.normalizarNumerosSemDenominador(a,b);
      //String [] s = Number.avaliaSinaisRealizaOperacaoAdicaoOuSubtracao(
      //    a.mantissa,b.mantissa, a.sinal, b.sinal);

      //System.out.println(s[0] + "\n" + s[1]);


      //System.out.println("subtrair: " + adicionar(a,subtrair(b,a)));
      //System.out.println("a: " + a);
      //System.out.println("b: " + b);
      //System.out.println("comparar: " + comparar(a,b));
      //Number c = new Number("0000-000200/e-15");
      //System.out.println("c:" + divisaoRacionalStrings("89998", "9", false,0));
      //System.out.println("c:" + divisaoRacionalStrings("9989", "999", false,7));
     // System.out.println("c:" + divisaoRacionalStrings("1000", "333", false,6));
    /*} catch (Exception ex) {
      System.out.println(ex.getMessage());
      ex.printStackTrace();
    }
  */

